VHs = {{25,7},{23,13},{6,18},{25,24},{14,26},{24,26}}
normal_wave_turns = {6,12,19}

-- *** Hex zones ***
-- Spawn zones
-- When spawning a set of units, the script will scan spawn zones hex by hex, in the order specified, and spawn units in free hexes it finds
-- If the script runs out of hexes, the remainder of the units will not be spawned.
sr_spawn_zoneNE = {{28,1},{29,2},{31,2},{32,2},{31,3},{32,4},{33,4},{32,5},{33,5},{33,6},{34,6},{34,8}}
sr_spawn_zoneSW = {{1,23},{2,25},{3,25},{4,25},{3,26},{5,26},{3,27},{4,27},{5,27},{6,28},{6,29},{7,30}}

-- *** Enemy waves ***
-- Each unit set is a list of blocks in the following format: { "<Unit ID>", "<Transport ID>", overstrength, experience }
-- If no organic transport is needed, its respective field should be empty string ("").

-- Spanish Republican waves
sr_wave1 = { {"SRMilitia", "", 0, 900}, {"SRMilitia", "", 0, 900}, {"TrubiaNaval", "", 0, 900}, {"UNL35", "", 0, 900}, {"SRInfantry", "", 0, 900}, {"SRInfantry", "", 0, 900}, {"SRInfantry", "", 0, 900}, {"I-15", "", 0, 900} }
sr_wave2 = { {"SRColonial", "", 0, 900}, {"SRColonial", "", 0, 900}, {"SRColonial", "", 0, 900}, {"T26", "", 0, 900}, {"T26", "", 0, 900}, {"PolikarpovI16", "", 0, 900} }
sr_wave3 = { {"SRInfantry", "", 0, 900}, {"SRColonial", "", 0, 900}, {"SREngineers", "", 0, 900}, {"T26", "", 0, 900}, {"TrubiaNaval", "", 0, 900}, {"BilbaoM1932", "", 0, 900}, {"I-15", "", 0, 900} }

-- There can be any number of waves in this list, the game will pick a random one every time
sr_waves = {sr_wave1, sr_wave2, sr_wave3}

-- There can be any number of zones in this list, the game will pick a random one every time
sr_zones = {sr_spawn_zoneNE, sr_spawn_zoneSW}

string_1 = NSLOCTEXT("scenario_CorunnaRoad", "string_1", "Nationalist prisoner liberated.  We're ready to transport them to safety.")

function Player1VH()
	local p1captured = 0
	for _,vh in ipairs(VHs) do
		local hex = world:GetHex(vh)
		if hex.owner == 0 or hex.owner == 1 then
			p1captured = p1captured + 1
		end
	end

	return p1captured
end

function Player1CustomSuccess(action)
	return action:GetActionClass() == "TurnsLimitAction" and Player1VH() > 3
end

function Player1CustomFail(action)
	return action:GetActionClass() == "TurnsLimitAction" and Player1VH() < 4
end


function SpawnWave(player, zone, units)
	local owner = world:GetPlayer(player)

	local cur_hex = 1

	-- Iterate through all units and spawn each one
	for _,u in ipairs(units) do
		-- Find the next vacant hex in the zone
		while zone[cur_hex] do
			hex = world:GetHex(zone[cur_hex])
			if hex:GetUnit(0) == nil then break end -- found
			cur_hex = cur_hex + 1
		end

		-- If we have run out of hexes in the zone, stop
		if not zone[cur_hex] then return end

		-- Create the unit in the free hex we've found
		-- 1. Create and execute purchase action, use unit type, transport type and overstrength as parameters
		local create_action = world:MakePurchaseAction(owner, u[1], u[2], u[3])
		create_action.auxiliary = false -- spawn core units
		create_action.cost = 0 -- don't take money for units we generate
		create_action.faction = owner.factions[1]  --set to first faction of owning player for created units
		world:Exec(create_action)

		-- 2. Create and execute deploy action, place the unit in the hex we've picked.
		local unit = world:GetUnit(create_action.id)
		deploy_action = world:MakeDeployAction(unit, zone[cur_hex])
		world:Exec(deploy_action)

		-- 3. Now modify the unit to set its experience to the value we need.
		unit.experience = u[4]
		
		-- 4. Now modify the unit to set its aggressiveness to the value we need.
		player_aggressiveness = { 0, 85, 70, 70 }
		unit.aggressiveness = player_aggressiveness[player+1]
		
	end

end

function SpawnWaveAUX(player, zone, units)
	local owner = world:GetPlayer(player)

	local cur_hex = 1

	local spawned_ids = {}

	-- Iterate through all units and spawn each one
	for _,u in ipairs(units) do
		-- Find the next vacant hex in the zone
		while zone[cur_hex] do
			hex = world:GetHex(zone[cur_hex])
			if hex:GetUnit(0) == nil then break end -- found
			cur_hex = cur_hex + 1
		end

		-- If we have run out of hexes in the zone, stop
		if not zone[cur_hex] then return end

		-- Create the unit in the free hex we've found
		-- 1. Create and execute purchase action, use unit type, transport type and overstrength as parameters
		local create_action = world:MakePurchaseAction(owner, u[1], u[2], u[3])
		create_action.auxiliary = true -- spawn AUX units
		create_action.cost = 0 -- don't take money for units we generate
		world:Exec(create_action)
		table.insert(spawned_ids, create_action.id)

		-- 2. Create and execute deploy action, place the unit in the hex we've picked.
		local unit = world:GetUnit(create_action.id)
		deploy_action = world:MakeDeployAction(unit, zone[cur_hex])
		world:Exec(deploy_action)

		-- 3. Now modify the unit to set its experience to the value we need.
		unit.experience = u[4]
		
		-- 4. Now modify the unit to set its aggressiveness to the value we need.
		player_aggressiveness = { 0, 85, 70, 70 }
		unit.aggressiveness = player_aggressiveness[player+1]
		
	end

	return spawned_ids
end

function OnNewRound(action)
	-- On the first call, initialize the random generator
	if world.round == 1 then
		math.randomseed(world.seed)
	end

	-- Do we need to spawn a normal wave?
	local is_normal_wave = IsItemInTable(normal_wave_turns, world.round)

	-- If we don't need to spawn any normal or boss waves this turn then just finish
	if not is_normal_wave then return end

	-- Determine the player to get the next wave
	-- Player 2 for Spanish Republican Units
	player = 2

	-- Determine the right zone and units to spawn
	local zone
	local units

	zone = sr_zones[ math.random( #sr_zones ) ] -- random normal SR zone
	units = sr_waves[ math.random( #sr_waves ) ] -- random normal SR wave

	-- Now spawn this wave!
	SpawnWave(player, zone, units)
end

-- *** Utility functions ***
function FindItemInTable(table, item)
	for index,i in ipairs(table) do
		if i == item then
			return index 
		end
	end

	return -1
end

function IsItemInTable(table, item)
	return FindItemInTable(table, item) ~= -1
end

function EmptyCondition()
	return true
end


function CreateAndAssignEnemyHero1(unit_id)
	local hero = NewHero()
	--hero.portrait = "/Game/Gui/Common/Heroes/DE/de_in_16.de_in_16"
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.name = NSLOCTEXT("scenario_CorunnaRoad", "EnemyHero", "")
	hero.extra_traits = {UnitTrait.Readiness, UnitTrait.DoubleAttack}
	-- hero.modifiers = {{type = Initiative, mod = 10}, {type = Spotting, mod = 1}}
	-- hero.attack_modifiers = {{type = TargetType.Soft, mod = 10}}
	-- hero.defense_modifiers = {{type = Ground, mod = 10}}
	local action = world:MakeNewHeroAction(2, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(world:GetUnit(unit_id), hero.id)
	world:Exec(assign_hero_action)
end

function EnemyHero()
	CreateAndAssignEnemyHero1(57) -- specify id of the unit which will get this hero
end


-- This table is initially empty, and will accumulate depots which we capture
-- Depots won't generate a bonus if we lose them and recapture for the second time
captured_depots = {}

Prison = {{6,3}}
escort_unit = -1
escort_zone = {{26,27},{27,27},{26,28},{27,28},{28,28},{26,29},{27,29}}

function OnCapturePrison(action)
	-- If this flag was not captured by player 0 or 1 (Allies), do nothing
	if(action.new_owner ~= 0 and action.new_owner ~= 1) then return end

	-- Don't generate reward if the same depot is captured for the second time
	if IsItemInTable(captured_depots, action.flag) then return end

	if IsItemInTable(Prison, action.flag) then	

		TutorialMessage(string_1)
		local player = world:GetPlayer(0)

		player = 0
		zone = { {6,1},{7,1},{8,1} }
		--testzone = { {25,29},{25,30},{26,30},{27,30} }
		units = { {"ESInfantry", "", -10, 0} }

		local spawned_ids = SpawnWaveAUX(player, zone, units)
		escort_unit = spawned_ids[1]
	else
		return
	end
	
	-- Remember that this depot has been captured
	table.insert(captured_depots, action.flag)
end


str = "Evakuierung erfolgreich"

message_shown = false
function EscortSuccess()
	if (escort_unit == -1) then return false end

	local unit = world:GetUnit(escort_unit)
	if (unit == nil) then return false end

	local success = IsItemInTable(escort_zone, unit.position)

	if (success) then
		if (not message_shown) then
			MessageBox(str)
			message_shown = true
		end			
	end

	return success
end

function EscortFailure()
	return (escort_unit ~= -1 and world:GetUnit(escort_unit) == nil)
end
