VHs = {{14,2},{25,2},{14,5},{17,10},{10,14},{21,14},{17,18},{11,24},{15,28},{9,29}}

normal_wave_turns = {10,13,17}

-- *** Hex zones ***
-- Spawn zones
-- When spawning a set of units, the script will scan spawn zones hex by hex, in the order specified, and spawn units in free hexes it finds
-- If the script runs out of hexes, the remainder of the units will not be spawned.
sr_spawn_zone = {{22,2},{22,3},{22,4},{22,5},{22,6},{23,6},{22,7},{23,7},{24,7},{25,7},{26,7},{23,8},{26,8}}

-- *** Enemy waves ***
-- Each unit set is a list of blocks in the following format: { "<Unit ID>", "<Transport ID>", overstrength, experience }
-- If no organic transport is needed, its respective field should be empty string ("").

-- Spanish Republican waves
sr_wave1 = { {"SRMilitia", "", 0, 900}, {"SRMilitia", "", 0, 900}, {"SRMilitia", "", 0, 900}, {"TrubiaNaval", "", 0, 900}, {"UNL35", "", 0, 900}, {"UNL35", "", 0, 900}, {"SRInfantry", "", 0, 900}, {"SRInfantry", "", 0, 900}, {"SRInfantry", "", 0, 900}, {"PolikarpovI16", "", 0, 900} }
sr_wave2 = { {"SRColonial", "", 0, 900}, {"SRColonial", "", 0, 900}, {"SRColonial", "", 0, 900}, {"SRColonial", "", 0, 900}, {"T26", "", 0, 900}, {"T26", "", 0, 900}, {"BT5", "", 0, 900}, {"BT5", "", 0, 900}, {"PolikarpovI16", "", 0, 900} }
sr_wave3 = { {"SRMilitia", "", 0, 900}, {"SRInfantry", "", 0, 900}, {"SRColonial", "", 0, 900}, {"SREngineers", "", 0, 900}, {"BA6sr", "", 0, 900}, {"T26", "", 0, 900}, {"BT5", "", 0, 900}, {"TrubiaNaval", "", 0, 900}, {"BilbaoM1932", "", 0, 900}, {"PolikarpovI16", "", 0, 900} }

-- There can be any number of waves in this list, the game will pick a random one every time
sr_waves = {sr_wave1, sr_wave2, sr_wave3}

string_1 = NSLOCTEXT("scenario_EarlyMadrid", "string_1", "<bi>Oberleutnant Wagner: </>Mein General, there are reports of large Republican reinforcements mustering within Madrid.  We might want to slow our advance to prepare for major counterattacks.")
string_2 = NSLOCTEXT("scenario_EarlyMadrid", "string_2", "The Verdeja unique tank has arrived from the South.")

function TimedMessageCondition()
	return world.round == 10 
end

function TimedMessageEffect()
	TutorialMessage(string_1)	
end

function Player1VH()
	local p1captured = 0
	for _,vh in ipairs(VHs) do
		local hex = world:GetHex(vh)
		if hex.owner == 0 or hex.owner == 1 then
			p1captured = p1captured + 1
		end
	end

	return p1captured
end

function Player1CustomSuccess(action)
	return action:GetActionClass() == "TurnsLimitAction" and Player1VH() > 5
end

function Player1CustomFail(action)
	return action:GetActionClass() == "TurnsLimitAction" and Player1VH() < 6
end

function SpawnWave(player, zone, units)
	local owner = world:GetPlayer(player)
	local cur_hex = 1

	-- Iterate through all units and spawn each one
	for _,u in ipairs(units) do
		-- Find the next vacant hex in the zone
		while zone[cur_hex] do
			hex = world:GetHex(zone[cur_hex])
			if hex:GetUnit(0) == nil then break end -- found
			cur_hex = cur_hex + 1
		end

		-- If we have run out of hexes in the zone, stop
		if not zone[cur_hex] then return end

		-- Create the unit in the free hex we've found
		-- 1. Create and execute purchase action, use unit type, transport type and overstrength as parameters
		local create_action = world:MakePurchaseAction(owner, u[1], u[2], u[3])
		create_action.auxiliary = false -- spawn core units
		create_action.cost = 0 -- don't take money for units we generate
		create_action.faction = owner.factions[1]  --set to first faction of owning player for created units
		world:Exec(create_action)

		-- 2. Create and execute deploy action, place the unit in the hex we've picked.
		local unit = world:GetUnit(create_action.id)
		deploy_action = world:MakeDeployAction(unit, zone[cur_hex])
		world:Exec(deploy_action)

		-- 3. Now modify the unit to set its experience to the value we need.
		unit.experience = u[4]
		
		-- 4. Now modify the unit to set its aggressiveness to the value we need.
		player_aggressiveness = { 0, 85, 70, 70 }
		unit.aggressiveness = player_aggressiveness[player+1]
	end
end

function OnNewRound(action)
	-- On the first call, initialize the random generator
	if world.round == 1 then
		math.randomseed(world.seed)
	end

	-- Do we need to spawn a normal wave?
	local is_normal_wave = IsItemInTable(normal_wave_turns, world.round)

	-- If we don't need to spawn any normal or boss waves this turn then just finish
	if not is_normal_wave then return end

	-- Determine the player to get the next wave
	-- Player 2 for Spanish Republican Units
	player = 2

	-- Determine the right zone and units to spawn
	local zone
	local units

		zone = sr_spawn_zone
		units = sr_waves[ math.random( #sr_waves ) ] -- random normal SR wave

	-- Now spawn this wave!
	SpawnWave(player, zone, units)

end

-- *** Utility functions ***
function FindItemInTable(table, item)
	for index,i in ipairs(table) do
		if i == item then
			return index 
		end
	end

	return -1
end

function IsItemInTable(table, item)
	return FindItemInTable(table, item) ~= -1
end

function EmptyCondition()
	return true
end

function Turn3Landing(action)
	TutorialMessage(string_2)
	player = 0
	zone = { {4,31},{5,31},{6,31},{7,31},{4,32},{5,32},{6,32},{7,32},{8,32},{9,32},{10,32} }
	units = { {"Verdeja2", "", 0, 0}  }

	SpawnWave(player, zone, units)
end

function IsThirdTurn()
	return world.round == 2
end

--Nemesis scripts
string_death = NSLOCTEXT("scenario_EarlyMadrid", "string_death", "<bi>Capitana Vega: </>Rest in peace, my misguided brother.")

function Condition2()
	local unit = world:GetUnit(64)
	return unit == nil 
end

function Effect2()
	if (campaign.nemesislives == 1) then
	TutorialMessage(string_death)
	campaign.nemesislives = 0
	end
end

function CreateAndAssignNemesis(unit_id)
	local hero = NewHero()
	--hero.portrait = "/Game/Gui/Common/Heroes/DE/de_in_16.de_in_16"
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.name = NSLOCTEXT("scenario_EarlyMadrid", "Nemesis", "")
	hero.extra_traits = {UnitTrait.Survivor, UnitTrait.Butcher, UnitTrait.TenaciousDefender}
	-- hero.modifiers = {{type = Initiative, mod = 10}, {type = Spotting, mod = 1}}
	-- hero.attack_modifiers = {{type = TargetType.Soft, mod = 10}}
	-- hero.defense_modifiers = {{type = Ground, mod = 10}}
	local action = world:MakeNewHeroAction(2, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(world:GetUnit(unit_id), hero.id)
	world:Exec(assign_hero_action)
end

function NemesisHero()
	CreateAndAssignNemesis(64) -- specify id of the unit which will get this hero
end

function CreateAndAssignEnemyHero1(unit_id)
	local hero = NewHero()
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.name = NSLOCTEXT("scenario_EarlyMadrid", "EnemyHero", "")
	hero.extra_traits = {UnitTrait.Unyielding}
	local action = world:MakeNewHeroAction(2, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(world:GetUnit(unit_id), hero.id)
	world:Exec(assign_hero_action)
end

function EnemyHero()
	CreateAndAssignEnemyHero1(43) -- specify id of the unit which will get this hero
end

function NemesisCheck()
	if (campaign.nemesislives == 0) then
	local nemesis = world:GetUnit(64)
	local death_action = world:MakeDeathAction(nemesis)
	world:Exec(death_action)
	end
end
