normal_wave_turns = {4,8,12,13}

-- *** Hex zones ***
-- Spawn zones
-- When spawning a set of units, the script will scan spawn zones hex by hex, in the order specified, and spawn units in free hexes it finds
-- If the script runs out of hexes, the remainder of the units will not be spawned.
sr_spawn_zoneN = {{16,1},{17,1},{18,1},{19,1},{21,1},{22,1},{23,2},{23,3},{24,3},{25,3}}
sr_spawn_zoneE = {{26,13},{27,13},{26,14},{27,14},{25,15},{26,15},{26,16},{27,16},{26,17},{27,17},{27,18},{28,18}}
sr_spawn_zoneS = {{15,26},{16,26},{17,26},{18,26},{12,27},{13,27},{15,27},{16,27},{17,27},{18,27},{14,28},{15,28},{16,28}}
sr_spawn_zoneSW = {{1,23},{2,23},{3,23},{3,24},{4,24},{4,25},{3,26},{4,26},{2,27},{3,27},{3,28}}

-- *** Enemy waves ***
-- Each unit set is a list of blocks in the following format: { "<Unit ID>", "<Transport ID>", overstrength, experience }
-- If no organic transport is needed, its respective field should be empty string ("").

-- Spanish Republican waves
sr_wave1 = { {"SRMilitia", "", 0, 900}, {"SRMilitia", "", 0, 900}, {"TrubiaNaval", "", 0, 900}, {"UNL35", "", 0, 900}, {"SRInfantry", "", 0, 900}, {"SRInfantry", "", 0, 900}, {"122mm M10-30", "HorseTransport", 0, 900}, {"I-15", "", 0, 900} }
sr_wave2 = { {"SRColonial", "", 0, 900}, {"SRColonial", "", 0, 900}, {"SRColonial", "", 0, 900}, {"T26", "", 0, 900}, {"T26", "", 0, 900}, {"152mmM10-30", "HorseTransport", 0, 900}, {"PolikarpovI16", "", 0, 900} }
sr_wave3 = { {"SRInfantry", "", 0, 900}, {"SRColonial", "", 0, 900}, {"SREngineers", "", 0, 900}, {"T26", "", 0, 900}, {"TrubiaNaval", "", 0, 900}, {"BilbaoM1932", "", 0, 900}, {"122mm M10-30", "HorseTransport", 0, 900}, {"I-15", "", 0, 900} }

-- There can be any number of waves in this list, the game will pick a random one every time
sr_waves = {sr_wave1, sr_wave2, sr_wave3}

-- There can be any number of zones in this list, the game will pick a random one every time
sr_zones = {sr_spawn_zoneN, sr_spawn_zoneE, sr_spawn_zoneS, sr_spawn_zoneSW}

function SpawnWave(player, zone, units)
	local owner = world:GetPlayer(player)

	local cur_hex = 1

	-- Iterate through all units and spawn each one
	for _,u in ipairs(units) do
		-- Find the next vacant hex in the zone
		while zone[cur_hex] do
			hex = world:GetHex(zone[cur_hex])
			if hex:GetUnit(0) == nil then break end -- found
			cur_hex = cur_hex + 1
		end

		-- If we have run out of hexes in the zone, stop
		if not zone[cur_hex] then return end

		-- Create the unit in the free hex we've found
		-- 1. Create and execute purchase action, use unit type, transport type and overstrength as parameters
		local create_action = world:MakePurchaseAction(owner, u[1], u[2], u[3])
		create_action.auxiliary = false -- spawn core units
		create_action.cost = 0 -- don't take money for units we generate
		create_action.faction = owner.factions[1]  --set to first faction of owning player for created units
		world:Exec(create_action)

		-- 2. Create and execute deploy action, place the unit in the hex we've picked.
		local unit = world:GetUnit(create_action.id)
		deploy_action = world:MakeDeployAction(unit, zone[cur_hex])
		world:Exec(deploy_action)

		-- 3. Now modify the unit to set its experience to the value we need.
		unit.experience = u[4]
		
		-- 4. Now modify the unit to set its aggressiveness to the value we need.
		player_aggressiveness = { 0, 85, 70, 70 }
		unit.aggressiveness = player_aggressiveness[player+1]
	end
end

function OnNewRound(action)
	-- On the first call, initialize the random generator
	if world.round == 1 then
		math.randomseed(world.seed)
	end

	-- Do we need to spawn a normal wave?
	local is_normal_wave = IsItemInTable(normal_wave_turns, world.round)

	-- Do we need to spawn a boss wave?
	--local boss_wave = FindItemInTable(boss_wave_turns, world.round) -- boss wave number
	--local is_boss_wave = (boss_wave ~= -1)

	-- If we don't need to spawn any normal or boss waves this turn then just finish
	if not is_normal_wave then return end

	-- Determine the player to get the next wave
	-- Player 2 for Spanish Republican Units
	player = 2

	-- Determine the right zone and units to spawn
	local zone
	local units

		zone = sr_zones[ math.random( #sr_zones ) ] -- random normal SR zone
		units = sr_waves[ math.random( #sr_waves ) ] -- random normal SR wave

	-- Now spawn this wave!
	SpawnWave(player, zone, units)

end

-- *** Utility functions ***
function FindItemInTable(table, item)
	for index,i in ipairs(table) do
		if i == item then
			return index 
		end
	end

	return -1
end

function IsItemInTable(table, item)
	return FindItemInTable(table, item) ~= -1
end

function EmptyCondition()
	return true
end
