normal_wave_turns = {3,5,8,10,13,17,20}

-- *** Hex zones ***
-- Spawn zones
-- When spawning a set of units, the script will scan spawn zones hex by hex, in the order specified, and spawn units in free hexes it finds
-- If the script runs out of hexes, the remainder of the units will not be spawned.
sr_spawn_zone1 = {{20,3},{21,3},{21,4},{22,4},{21,5},{22,5},{22,6},{23,6},{22,7},{23,7},{22,8},{24,8},{21,9},{22,9},{23,9}}
sr_spawn_zone2 = {{22,17},{23,17},{24,17},{22,18},{24,18},{22,19},{23,19},{24,19},{23,20},{24,20},{25,20},{23,21},{24,21},{25,21}}
sr_spawn_zone3 = {{23,25},{24,25},{25,25},{24,26},{25,26},{22,27},{23,27},{22,28},{23,28},{24,28}}
sr_spawn_zone4 = {{21,35},{22,35},{23,35},{21,36},{22,36},{23,36},{20,37},{21,37},{22,37},{21,38},{22,38},{21,39},{22,39}}

-- *** Enemy waves ***
-- Each unit set is a list of blocks in the following format: { "<Unit ID>", "<Transport ID>", overstrength, experience }
-- If no organic transport is needed, its respective field should be empty string ("").

-- Spanish Republican waves
sr_wave1 = { {"SRMilitia", "HorseTransport", 0, 900}, {"SRMilitia", "HorseTransport", 0, 900}, {"TrubiaNaval", "", 3, 900}, {"UNL35", "", 0, 900}, {"SRInfantry", "HorseTransport", 5, 900}, {"SRInfantry", "HorseTransport", 0, 900}, {"122mm M10-30", "HorseTransport", 0, 900}, {"I-15", "", 2, 900}, {"I-15", "", 2, 900} }
sr_wave2 = { {"SRColonial", "HorseTransport", 4, 900}, {"SRColonial", "HorseTransport", 0, 900}, {"SRColonial", "HorseTransport", 0, 900}, {"T26", "", 0, 900}, {"T26", "", 0, 900}, {"152mmM10-30", "HorseTransport", 0, 900}, {"PolikarpovI16", "", 2, 2900}, {"I-15", "", 5, 900} }
sr_wave3 = { {"SRInfantry", "HorseTransport", 0, 900}, {"SRColonial", "HorseTransport", 0, 900}, {"SREngineers", "HorseTransport", 4, 900}, {"T26", "", 0, 900}, {"TrubiaNaval", "", 0, 900}, {"BilbaoM1932", "", 5, 900}, {"122mm M10-30", "HorseTransport", 0, 900}, {"I-15", "", 0, 900}, {"PolikarpovI16", "", 0, 900} }

-- There can be any number of waves in this list, the game will pick a random one every time
sr_waves = {sr_wave1, sr_wave2, sr_wave3}

-- There can be any number of zones in this list, the game will pick a random one every time
sr_zones = {sr_spawn_zone1, sr_spawn_zone2, sr_spawn_zone3, sr_spawn_zone4}

function Player1VH()
	local p1captured = 0
	for _,vh in ipairs(VHs) do
		local hex = world:GetHex(vh)
		if hex.owner == 0 or hex.owner == 1 then
			p1captured = p1captured + 1
		end
	end
	return p1captured
end

function Player1CustomSuccess(action)
	return action:GetActionClass() == "TurnsLimitAction" and Player1VH() > 4
end

function Player1CustomFail(action)
	return action:GetActionClass() == "TurnsLimitAction" and Player1VH() < 5
end

function SpawnWave(player, zone, units)
	local owner = world:GetPlayer(player)

	local cur_hex = 1

	-- Iterate through all units and spawn each one
	for _,u in ipairs(units) do
		-- Find the next vacant hex in the zone
		while zone[cur_hex] do
			hex = world:GetHex(zone[cur_hex])
			if hex:GetUnit(0) == nil then break end -- found
			cur_hex = cur_hex + 1
		end

		-- If we have run out of hexes in the zone, stop
		if not zone[cur_hex] then return end

		-- Create the unit in the free hex we've found
		-- 1. Create and execute purchase action, use unit type, transport type and overstrength as parameters
		local create_action = world:MakePurchaseAction(owner, u[1], u[2], u[3])
		create_action.auxiliary = false -- spawn core units
		create_action.cost = 0 -- don't take money for units we generate
		create_action.faction = owner.factions[1]  --set to first faction of owning player for created units
		world:Exec(create_action)

		-- 2. Create and execute deploy action, place the unit in the hex we've picked.
		local unit = world:GetUnit(create_action.id)
		deploy_action = world:MakeDeployAction(unit, zone[cur_hex])
		world:Exec(deploy_action)

		-- 3. Now modify the unit to set its experience to the value we need.
		unit.experience = u[4]
		
		-- 4. Now modify the unit to set its aggressiveness to the value we need.
		player_aggressiveness = { 0, 85, 70, 70 }
		unit.aggressiveness = player_aggressiveness[player+1]
	end
end

function OnNewRound(action)
	-- On the first call, initialize the random generator
	if world.round == 1 then
		math.randomseed(world.seed)
	end

	-- Do we need to spawn a normal wave?
	local is_normal_wave = IsItemInTable(normal_wave_turns, world.round)

	-- If we don't need to spawn any normal or boss waves this turn then just finish
	if not is_normal_wave then return end

	-- Determine the player to get the next wave
	-- Player 2 for Spanish Republican Units
	player = 2

	-- Determine the right zone and units to spawn
	local zone
	local units

	zone = sr_zones[ math.random( #sr_zones ) ] -- random normal SR zone
	units = sr_waves[ math.random( #sr_waves ) ] -- random normal SR wave

	-- Now spawn this wave!
	SpawnWave(player, zone, units)
end

-- *** Utility functions ***
function FindItemInTable(table, item)
	for index,i in ipairs(table) do
		if i == item then
			return index 
		end
	end
	return -1
end

function IsItemInTable(table, item)
	return FindItemInTable(table, item) ~= -1
end

function EmptyCondition()
	return true
end


--Nemesis scripts
string_death = NSLOCTEXT("scenario_BattleoftheEbro", "string_death", "<bi>Capitana Vega: </>Rest in peace, my misguided brother.")

function Condition2()
	local unit = world:GetUnit(145)
	return unit == nil 
end

function Effect2()
	if (campaign.nemesislives == 1) then
	TutorialMessage(string_death)
	campaign.nemesislives = 0
	end
end

function NemesisCheck()

if (campaign.nemesislives == 0) then
   local nemesis = world:GetUnit(145)
   local death_action = world:MakeDeathAction(nemesis)
   world:Exec(death_action)
	end
end

function CreateAndAssignNemesis(unit_id)
	local hero = NewHero()
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.name = NSLOCTEXT("scenario_BattleoftheEbro", "Nemesis", "")
	hero.extra_traits = {UnitTrait.IgnoresZOC, UnitTrait.Survivor, UnitTrait.Butcher, UnitTrait.TenaciousDefender}
	local action = world:MakeNewHeroAction(2, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(world:GetUnit(unit_id), hero.id)
	world:Exec(assign_hero_action)
end

function NemesisHero()
	CreateAndAssignNemesis(145) -- specify id of the unit which will get this hero
end


--4 enemy heroes
function CreateAndAssignEnemyHero1(unit_id)
	local hero = NewHero()
	--hero.portrait = "/Game/Gui/Common/Heroes/DE/de_in_16.de_in_16"
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.name = NSLOCTEXT("scenario_BattleoftheEbro", "EnemyHero", "")
	hero.extra_traits = {UnitTrait.AggressiveCounterattack}
	local action = world:MakeNewHeroAction(2, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(world:GetUnit(unit_id), hero.id)
	world:Exec(assign_hero_action)
end

function EnemyHero()
	CreateAndAssignEnemyHero1(66) -- specify id of the unit which will get this hero
	CreateAndAssignEnemyHero2(98) -- specify id of the unit which will get this hero
	CreateAndAssignEnemyHero3(129) -- specify id of the unit which will get this hero
	CreateAndAssignEnemyHero4(72) -- specify id of the unit which will get this hero
end

function CreateAndAssignEnemyHero2(unit_id)
	local hero = NewHero()
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.name = NSLOCTEXT("scenario_BattleoftheEbro", "EnemyHero", "")
	hero.extra_traits = {UnitTrait.Camouflage}
	local action = world:MakeNewHeroAction(2, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(world:GetUnit(unit_id), hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignEnemyHero3(unit_id)
	local hero = NewHero()
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.name = NSLOCTEXT("scenario_BattleoftheEbro", "EnemyHero", "")
	hero.extra_traits = {UnitTrait.OnTheRoll, UnitTrait.LastStand}
	local action = world:MakeNewHeroAction(2, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(world:GetUnit(unit_id), hero.id)
	world:Exec(assign_hero_action)
end

function CreateAndAssignEnemyHero4(unit_id)
	local hero = NewHero()
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.name = NSLOCTEXT("scenario_BattleoftheEbro", "EnemyHero", "")
	hero.extra_traits = {UnitTrait.Leadership, UnitTrait.LastStand}
	local action = world:MakeNewHeroAction(2, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(world:GetUnit(unit_id), hero.id)
	world:Exec(assign_hero_action)
end
