VHs = {{19,18}}

normal_wave_turns = {9}

-- *** Hex zones ***
-- Spawn zones
-- When spawning a set of units, the script will scan spawn zones hex by hex, in the order specified, and spawn units in free hexes it finds
-- If the script runs out of hexes, the remainder of the units will not be spawned.
sr_spawn_zoneS = {{27,18},{28,18},{26,19},{27,19},{26,20},{27,20},{28,20},{25,21},{26,21},{27,21},{25,22},{26,22},{27,22},{24,23},{25,23},{26,23},{24,24},{25,24}}

-- *** Enemy waves ***
-- Each unit set is a list of blocks in the following format: { "<Unit ID>", "<Transport ID>", overstrength, experience }
-- If no organic transport is needed, its respective field should be empty string ("").

-- Spanish Republican waves
sr_wave1 = { {"SRMilitia", "", 0, 900}, {"SRMilitia", "", 0, 900}, {"TrubiaNaval", "", 2, 900}, {"UNL35", "", 0, 900}, {"SRInfantry", "", 1, 900}, {"SRInfantry", "", 1, 900}, {"122mm M10-30", "HorseTransport", 1, 900}, {"SRColonial", "", 5, 2900}, {"SRColonial", "", 2, 900}, {"SRColonial", "", 2, 900}, {"T26", "", 2, 900}, {"T26", "", 2, 900}, {"152mmM10-30", "HorseTransport", 2, 900} }

-- There can be any number of waves in this list, the game will pick a random one every time
sr_waves = {sr_wave1}

-- There can be any number of zones in this list, the game will pick a random one every time
sr_zones = {sr_spawn_zoneS}


function Player1VH()
	local p1captured = 0
	for _,vh in ipairs(VHs) do
		local hex = world:GetHex(vh)
		if hex.owner == 0 or hex.owner == 1 then
			p1captured = p1captured + 1
		end
	end
	return p1captured
end

function Player1CustomSuccess(action)
	return action:GetActionClass() == "TurnsLimitAction" and Player1VH() > 0
end

function Player1CustomFail(action)
	return action:GetActionClass() == "TurnsLimitAction" and Player1VH() < 1
end


function SpawnWave(player, zone, units)
	local owner = world:GetPlayer(player)
	local cur_hex = 1

	-- Iterate through all units and spawn each one
	for _,u in ipairs(units) do
		-- Find the next vacant hex in the zone
		while zone[cur_hex] do
			hex = world:GetHex(zone[cur_hex])
			if hex:GetUnit(0) == nil then break end -- found
			cur_hex = cur_hex + 1
		end

		-- If we have run out of hexes in the zone, stop
		if not zone[cur_hex] then return end

		-- Create the unit in the free hex we've found
		-- 1. Create and execute purchase action, use unit type, transport type and overstrength as parameters
		local create_action = world:MakePurchaseAction(owner, u[1], u[2], u[3])
		create_action.auxiliary = false -- spawn core units
		create_action.cost = 0 -- don't take money for units we generate
		create_action.faction = owner.factions[1]  --set to first faction of owning player for created units
		world:Exec(create_action)

		-- 2. Create and execute deploy action, place the unit in the hex we've picked.
		local unit = world:GetUnit(create_action.id)
		deploy_action = world:MakeDeployAction(unit, zone[cur_hex])
		world:Exec(deploy_action)

		-- 3. Now modify the unit to set its experience to the value we need.
		unit.experience = u[4]
		
		-- 4. Now modify the unit to set its aggressiveness to the value we need.
		player_aggressiveness = { 0, 85, 70, 70 }
		unit.aggressiveness = player_aggressiveness[player+1]
		
	end
end

function OnNewRound(action)
	-- On the first call, initialize the random generator
	if world.round == 1 then
		math.randomseed(world.seed)
	end

	-- Do we need to spawn a normal wave?
	local is_normal_wave = IsItemInTable(normal_wave_turns, world.round)

	-- If we don't need to spawn any normal or boss waves this turn then just finish
	if not is_normal_wave then return end

	-- Determine the player to get the next wave
	-- Player 2 for Spanish Republican Units
	player = 2

	-- Determine the right zone and units to spawn
	local zone
	local units

	zone = sr_zones[ math.random( #sr_zones ) ] -- random normal SR zone
	units = sr_waves[ math.random( #sr_waves ) ] -- random normal SR wave

	-- Now spawn this wave!
	SpawnWave(player, zone, units)
end

-- *** Utility functions ***
function FindItemInTable(table, item)
	for index,i in ipairs(table) do
		if i == item then
			return index 
		end
	end

	return -1
end

function IsItemInTable(table, item)
	return FindItemInTable(table, item) ~= -1
end

function AlwaysTrue()
	return true
end


function CreateAndAssignEnemyHero1(unit_id)
	local hero = NewHero()
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.name = "EnemyHero1"
	hero.extra_traits = {UnitTrait.DoubleAttack}
	local action = world:MakeNewHeroAction(2, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(world:GetUnit(unit_id), hero.id)
	world:Exec(assign_hero_action)
end

function EnemyHero()
	CreateAndAssignEnemyHero1(17) -- specify id of the unit which will get this hero
end

function CreateAndAssignEnemyHero2(unit_id)
	local hero = NewHero()
	hero.portrait = "/Game/Gui/Common/Heroes/generic_standin.generic_standin"
	hero.name = "EnemyHero2"
	hero.extra_traits = {UnitTrait.ShockTactics}
	local action = world:MakeNewHeroAction(2, hero)
	world:Exec(action)

	local assign_hero_action = world:MakeAssignHeroAction(world:GetUnit(unit_id), hero.id)
	world:Exec(assign_hero_action)
end

function EnemyHero2()
	CreateAndAssignEnemyHero2(18) -- specify id of the unit which will get this hero
end


-- escort scripts
escort_zone = {{2,5},{3,5},{2,6},{3,6},{4,6},{2,7},{3,7}}
escort_unit = 50
str = "Evakuierung erfolgreich"

message_shown = false
function EscortSuccess()
	local unit = world:GetUnit(50)
	if (unit == nil) then return false end
	local success = IsItemInTable(escort_zone, unit.position)
	if (success) then
		if (not message_shown) then
			MessageBox(str)
			message_shown = true
		end			
	end
	return success
end

function EscortFailure()
	return (escort_unit ~= -1 and world:GetUnit(escort_unit) == nil)
end
